@@ -1,7 +1,6 @@
 package Gimp::Fu;
 
-use Carp;
-use Gimp ();
+use Gimp 'croak';
 use Gimp::Data;
 use File::Basename;
 
@@ -133,6 +132,11 @@ sub import {
    }
 }
 
+sub carp {
+   require Carp;
+   goto &Carp::carp;
+}
+
 # expand all the pod directives in string (currently they are only removed)
 sub expand_podsections() {
    my $pod;
@@ -644,6 +648,7 @@ sub this_script {
    my @names;
    for my $this (@scripts) {
       my $fun = (split /\//,$this->[1])[-1];
+      $fun =~ s/^(?:perl_fu|plug_in)_//;
       return $this if lc($exe) eq lc($fun);
       push(@names,$fun);
    }
@@ -693,7 +698,7 @@ sub mangle_key {
    $key;
 }
 
-sub net {
+Gimp::on_net {
    no strict 'refs';
    my $this = this_script;
    my(%map,@args);
@@ -742,13 +747,20 @@ sub net {
    $this->[0]->($interact>0 ? $this->[7]=~/^<Image>/ ? (&Gimp::RUN_FULLINTERACTIVE,undef,undef,@args)
                                                      : (&Gimp::RUN_INTERACTIVE,@args)
                             : (&Gimp::RUN_NONINTERACTIVE,@args));
-}
+};
 
 # the <Image> arguments
 @image_params = ([&Gimp::PARAM_IMAGE	, "image",	"The image to work on"],
                  [&Gimp::PARAM_DRAWABLE	, "drawable",	"The drawable to work on"]);
 
-sub query {
+@load_params  = ([&Gimp::PARAM_STRING	, "filename",	"The name of the file"],
+                 [&Gimp::PARAM_STRING	, "raw_filename","The name of the file"]);
+
+@save_params  = (@image_params, @load_params);
+
+@load_retvals = ([&Gimp::PARAM_IMAGE	, "image",	"Output image"]);
+
+Gimp::on_query {
    my($type);
    expand_podsections;
    script:
@@ -763,12 +775,20 @@ sub query {
       if ($menupath=~/^<Image>\//) {
          $type=&Gimp::PROC_PLUG_IN;
          unshift(@$params,@image_params);
+      } elsif ($menupath=~/^<Load>\//) {
+         $type=&Gimp::PROC_PLUG_IN;
+         unshift(@$params,@load_params);
+         unshift(@$results,@load_retvals);
+      } elsif ($menupath=~/^<Save>\//) {
+         $type=&Gimp::PROC_PLUG_IN;
+         unshift(@$params,@save_params);
       } elsif ($menupath=~/^<Toolbox>\//) {
          $type=&Gimp::PROC_EXTENSION;
       } elsif ($menupath=~/^<None>/) {
          $type=&Gimp::PROC_EXTENSION;
+         $menupath=undef;
       } else {
-         die "menupath _must_ start with <Image>, <Toolbox> or <None>!";
+         die "menupath _must_ start with <Image>, <Toolbox>, <Load>, <Save> or <None>!";
       }
       
       unshift(@$params,
@@ -794,7 +814,7 @@ sub query {
 
       Gimp::logger(message => 'OK', function => $function, fatal => 0);
    }
-}
+};
 
 =cut
 
@@ -824,9 +844,10 @@ sub query {
 
 The pdb name of the function, i.e. the name under which is will be
 registered in the Gimp database. If it doesn't start with "perl_fu_",
-"plug_in_" or "extension_", it will be prepended. If you don't want this,
-prefix your function name with a single "+". The idea here is that every
-Gimp::Fu plug-in will be found under the common C<perl_fu_>-prefix.
+"file_", "plug_in_" or "extension_", it will be prepended. If you
+don't want this, prefix your function name with a single "+". The idea
+here is that every Gimp::Fu plug-in will be found under the common
+C<perl_fu_>-prefix.
 
 =item blurb
 
@@ -1053,7 +1074,7 @@ sub register($$$$$$$$$;@) {
 
    $function=~/^[0-9a-z_]+(-ALT)?$/ or carp "$function: function name contains unusual characters, good style is to use only 0-9, a-z and _";
    
-   $function="perl_fu_".$function unless $function=~/^(?:perl_fu|extension|plug_in)/ || $function=~s/^\+//;
+   $function="perl_fu_".$function unless $function=~/^(?:\+|perl_fu_|extension_|plug_in_|file_)/;
    
    Gimp::logger message => "function name contains dashes instead of underscores",
                 function => $function, fatal => 0
@@ -1066,10 +1087,16 @@ sub register($$$$$$$$$;@) {
       if ($menupath=~/^<Image>\//) {
          @_ >= 2 or die "<Image> plug-in called without both image and drawable arguments!\n";
          @pre = (shift,shift);
-      } elsif ($menupath=~/^<Toolbox>\//) {
+      } elsif ($menupath=~/^<Toolbox>\// or !defined $menupath) {
          # valid ;)
+      } elsif ($menupath=~/^<Load>\//) {
+         @_ >= 2 or die "<Load> plug-in called without the 5 standard arguments!\n";
+         @pre = (shift,shift);
+      } elsif ($menupath=~/^<Save>\//) {
+         @_ >= 4 or die "<Save> plug-in called without the 5 standard arguments!\n";
+         @pre = (shift,shift,shift,shift);
       } else {
-         die "menupath _must_ start with <Image> or <Toolbox>!";
+         die "menupath _must_ start with <Image>, <Toolbox>, <Load> or <Save>!";
       }
       
       if (@defaults) {
@@ -1121,7 +1148,7 @@ sub register($$$$$$$$$;@) {
       my @imgs = &$code(@pre,@_);
       $old_trace = Gimp::set_trace (0);
       
-      if (@imgs) {
+      if (@imgs && $menupath !~ /^<Load>\//) {
          for my $i (0..$#imgs) {
             my $img = $imgs[$i];
             next unless defined $img;
@@ -1137,12 +1164,12 @@ sub register($$$$$$$$$;@) {
                } elsif ($run_mode != &Gimp::RUN_NONINTERACTIVE) {
                   $img->display_new unless $input_image && $$img == $$input_image;
                }
-            } elsif (!@$results) {
+            } elsif (!@$retvals) { 
                warn "WARNING: $function returned something that is not an image: \"$img\"\n";
             }
 	 }
+         Gimp->displays_flush;
       }
-      Gimp->displays_flush;
       
       Gimp::set_trace ($old_trace);
       wantarray ? @imgs : $imgs[0];
