@@ -49,10 +49,10 @@ require      Exporter;
                );
 #@EXPORT_OK = qw();
 
-$VERSION=$Gimp::VERSION;
-
 use Gimp;
 
+$VERSION=$Gimp::VERSION;
+
 ##############################################################################
 =pod
 
@@ -250,13 +250,13 @@ sub gimp_text_wh {
 
 =pod
 
-=item C<gimp_drawable_mask $drawable>
+=item C<gimp_drawable_bounds $drawable>
 
 returns an array (x,y,w,h) containing the upper left corner and the size of the
 current mask, just as needed by pixelrgn and similar functions.
 
 =cut
-sub gimp_drawable_mask {
+sub gimp_drawable_bounds {
    my @b = (shift->mask_bounds)[1..4];
    (@b[0,1],$b[2]-$b[0],$b[3]-$b[1]);
 }
@@ -268,6 +268,11 @@ sub gimp_drawable_mask {
 returns the corresponding layer type for an image, alpha controls wether the layer type
 is with alpha or not. Example: imagetype: RGB -> RGB_IMAGE (or RGBA_IMAGE).
 
+=item C<gimp_layer2imagetype $layertype>
+
+returns the corresponding layer type for an image, alpha controls wether the layer type
+is with alpha or not. Example: imagetype: RGB -> RGB_IMAGE (or RGBA_IMAGE).
+
 =item C<gimp_image_add_new_layer $image,$index,$fill_type,$alpha>
 
 creates a new layer and adds it at position $index (default 0) to the
@@ -296,6 +301,17 @@ sub gimp_image_layertype {
    die;
 }
 
+sub gimp_layer2imagetype {
+   my $type = shift;
+   $type == RGB_IMAGE		? RGB		:
+   $type == RGBA_IMAGE		? RGB		:
+   $type == GRAY_IMAGE		? GRAY		:
+   $type == GRAYA_IMAGE		? GRAY		:
+   $type == INDEXED_IMAGE	? INDEXED	:
+   $type == INDEXEDA_IMAGE	? INDEXED	:
+   die;
+}
+
 sub gimp_image_add_new_layer {
    my ($image,$index,$filltype,$alpha)=@_;
    my $layer = new Layer ($image, $image->width, $image->height, $image->layertype (defined $alpha ? $alpha : 1), join(":",(caller)[1,2]), 100, NORMAL_MODE);
@@ -326,7 +342,8 @@ sub gimp_layer_get_position {
    my $layer = shift;
    my @layers = $layer->image->get_layers;
    for (0..$#layers) {
-      return $_ if ${$layers[$_]} == $$layer;
+      # the my is necessary for broken perl (return $_ => undef)
+      return (my $index=$_) if ${$layers[$_]} == $$layer;
    }
    ();
 }
